import uuid

import pytest
from pydantic_forms.exceptions import FormValidationError

from gso.products.product_types.vrf import VRF
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.workflow()
def test_modify_vrf_router_list(vrf_subscription_factory, router_subscription_factory, faker):
    subscription_id = vrf_subscription_factory()
    initial_vrf_data = [
        {"subscription_id": subscription_id},
        {
            "router_list": [{"router_id": router_subscription_factory()}, {"router_id": router_subscription_factory()}],
        },
    ]
    result, _, _ = run_workflow("modify_vrf_router_list", initial_vrf_data)
    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = VRF.from_subscription(subscription_id)
    assert subscription.status == "active"
    assert len(subscription.vrf.vrf_router_list) == 2


@pytest.mark.workflow()
def test_modify_vrf_router_list_with_invalid_router_id(vrf_subscription_factory, router_subscription_factory, faker):
    subscription_id = vrf_subscription_factory()
    initial_vrf_data = [
        {"subscription_id": subscription_id},
        {
            "router_list": [{"router_id": uuid.uuid4()}],
        },
    ]

    with pytest.raises(FormValidationError, match="Input should be an instance of Select a router"):
        run_workflow("modify_vrf_router_list", initial_vrf_data)


@pytest.mark.workflow()
def test_modify_vrf_router_list_with_duplicate_router_id(vrf_subscription_factory, router_subscription_factory, faker):
    subscription_id = vrf_subscription_factory()
    router_id = router_subscription_factory()
    initial_vrf_data = [
        {"subscription_id": subscription_id},
        {
            "router_list": [{"router_id": router_id}, {"router_id": router_id}],
        },
    ]

    with pytest.raises(FormValidationError, match="List must be unique"):
        run_workflow("modify_vrf_router_list", initial_vrf_data)
